@@ -5,11 +5,11 @@ import xml.etree.ElementTree as ET
 from collections import defaultdict
 from time import sleep
 from jenkinsapi.build import Build
+from jenkinsapi.invocation import Invocation
 from jenkinsapi.jenkinsbase import JenkinsBase
-from jenkinsapi import exceptions
+from jenkinsapi.queue import QueueItem
 from jenkinsapi.mutable_jenkins_thing import MutableJenkinsThing
-
-from jenkinsapi.exceptions import NoBuildData, NotFound, NotInQueue, WillNotBuild
+from jenkinsapi.exceptions import NoBuildData, NotFound, NotInQueue, WillNotBuild, UnknownQueueItem
 
 log = logging.getLogger(__name__)
 
@@ -92,56 +92,61 @@ class Job(JenkinsBase, MutableJenkinsThing):
         assert isinstance(block, bool)
         assert isinstance(skip_if_running, bool)
 
+        # Create a new invocation instance
+        invocation = Invocation(self)
+
         # Either copy the params dict or make a new one.
         build_params = build_params and dict(
             build_params.items()) or {}  # Via POSTed JSON
         params = {}  # Via Get string
 
-        if self.is_queued():
-            raise WillNotBuild('%s is already queued' % repr(self))
+        with invocation:
+            if self.is_queued():
+                raise WillNotBuild('%s is already queued' % repr(self))
 
-        elif self.is_running():
-            if skip_if_running:
-                log.warn(
-                    "Will not request new build because %s is already running", self.name)
-            else:
-                log.warn(
-                    "Will re-schedule %s even though it is already running", self.name)
+            elif self.is_running():
+                if skip_if_running:
+                    log.warn(
+                        "Will not request new build because %s is already running", self.name)
+                else:
+                    log.warn(
+                        "Will re-schedule %s even though it is already running", self.name)
 
-        log.info("Attempting to start %s on %s", self.name, repr(
-            self.get_jenkins_obj()))
+            log.info("Attempting to start %s on %s", self.name, repr(
+                self.get_jenkins_obj()))
 
-        url = self.get_build_triggerurl()
+            url = self.get_build_triggerurl()
 
-        if cause:
-            build_params['cause'] = cause
+            if cause:
+                build_params['cause'] = cause
 
-        if securitytoken:
-            params['token'] = securitytoken
+            if securitytoken:
+                params['token'] = securitytoken
 
-        response = self.jenkins.requester.post_and_confirm_status(
-            url,
-            data={'json': self.mk_json_from_build_parameters(
-                build_params)},  # See above - build params have to be JSON encoded & posted.
-            params=params,
-            valid=[200, 201]
-        )
-        if invoke_pre_check_delay > 0:
-            log.info(
-                "Waiting for %is to allow Jenkins to catch up", invoke_pre_check_delay)
-            sleep(invoke_pre_check_delay)
-        if block:
-            total_wait = 0
-
-            while self.is_queued():
+            response = self.jenkins.requester.post_and_confirm_status(
+                url,
+                data={'json': self.mk_json_from_build_parameters(
+                    build_params)},  # See above - build params have to be JSON encoded & posted.
+                params=params,
+                valid=[200, 201]
+            )
+            if invoke_pre_check_delay > 0:
                 log.info(
-                    "Waited %is for %s to begin...", total_wait, self.name)
-                sleep(invoke_block_delay)
-                total_wait += invoke_block_delay
-            if self.is_running():
-                running_build = self.get_last_build()
-                running_build.block_until_complete(
-                    delay=invoke_pre_check_delay)
+                    "Waiting for %is to allow Jenkins to catch up", invoke_pre_check_delay)
+                sleep(invoke_pre_check_delay)
+            if block:
+                total_wait = 0
+
+                while self.is_queued():
+                    log.info(
+                        "Waited %is for %s to begin...", total_wait, self.name)
+                    sleep(invoke_block_delay)
+                    total_wait += invoke_block_delay
+                if self.is_running():
+                    running_build = self.get_last_build()
+                    running_build.block_until_complete(
+                        delay=invoke_pre_check_delay)
+        return invocation
 
     def _buildid_for_type(self, buildtype):
         self.poll()
@@ -283,6 +288,14 @@ class Job(JenkinsBase, MutableJenkinsThing):
         self.poll()
         return self._data["inQueue"]
 
+    def get_queue_item(self):
+        """
+        Return a QueueItem if this object is in a queue, otherwise raise an exception
+        """
+        if not self.is_queued():
+            raise UnknownQueueItem()
+        return QueueItem(self.jenkins, **self._data['queueItem'])
+
     def is_running(self):
         self.poll()
         try:
